------------------------------------------------------
-- DisenchantPredictor.lua
-- Version 1600.3
------------------------------------------------------

Hilite = GFWUtils.Hilite;

-- Configuration
FDP_Config = { };
FDP_Config.Tooltip = true;
FDP_Config.Verbose = true;

-- formula for dust by level: max(1, floor((level - 11) / 10))
FDP_DUST_LIST = {
	"Strange Dust",		-- 1-20
	"Soul Dust",		-- 21-30
	"Vision Dust",		-- 31-40
	"Dream Dust",		-- 41-50
	"Illusion Dust",	-- 51-60
}

-- formula for essence by level: max(1, floor((level - 6) / 5))
FDP_ESSENCE_LIST = {
	"Lesser Magic Essence",		-- 1-10
	"Greater Magic Essence",	-- 11-15
	"Lesser Astral Essence",	-- 16-20
	"Greater Astral Essence",	-- 21-25
	"Lesser Mystic Essence",	-- 26-30
	"Greater Mystic Essence",	-- 31-35
	"Lesser Nether Essence",	-- 36-40
	"Greater Nether Essence",	-- 41-45
	"Lesser Eternal Essence",	-- 46-50
	"Greater Eternal Essence",	-- 51-55 (51-60 is the same)
	"Greater Eternal Essence",	-- 56-60 (51-60 is the same)
}
	
-- formula for shard by level: max(1, floor((level - 16) / 5))
FDP_SHARD_LIST = {
	"Small Glimmering Shard",	-- 1-20
	"Large Glimmering Shard",	-- 21-25
	"Small Glowing Shard",		-- 26-30
	"Large Glowing Shard",		-- 31-35
	"Small Radiant Shard",		-- 36-40
	"Large Radiant Shard",		-- 41-45
	"Small Brilliant Shard",	-- 46-50
	"Large Brilliant Shard",	-- 51-55 (51-60 is the same)
	"Large Brilliant Shard",	-- 56-60 (51-60 is the same)
}

FDP_DUST_LEVELS = {
	"1-20", "21-30", "31-40", "41-50", "51-60",
}

FDP_ESSENCE_LEVELS = {
	"1-10", "11-15", "16-20", "21-25", "26-30", "31-35", "36-40", "41-45", "46-50", "51-60", "51-60",
}
	
FDP_SHARD_LEVELS = {
	"1-20", "21-25", "26-30", "31-35", "36-40", "41-45", "46-50", "51-60", "51-60",
}

FDP_COLOR_EPIC = "ffa335ee";
FDP_COLOR_RARE = "ff0070dd";
FDP_COLOR_UNCOMMON = "ff1eff00";

local function FDP_Tooltip_Hook(frame, name, link, count, data)
	FDP_Old_Tooltip_Hook(frame, name, link, count, data);

	if ( FDP_Config.Tooltip ) then
		local kind = nil;
		if (string.find(name, "^%w+ Dust$")) then
			kind = "dust";
			levelRange = FDP_DUST_LEVELS[GFWTable.IndexOf(FDP_DUST_LIST, name)];
		elseif (string.find(name, "^Lesser %w+ Essence$") or string.find(name, "^Greater %w+ Essence$")) then
			kind = "essence";
			levelRange = FDP_ESSENCE_LEVELS[GFWTable.IndexOf(FDP_ESSENCE_LIST, name)];
		elseif (string.find(name, "^Small %w+ Shard$") or string.find(name, "^Large %w+ Shard$")) then
			kind = "shard";
			levelRange = FDP_SHARD_LEVELS[GFWTable.IndexOf(FDP_SHARD_LIST, name)];
		end
		
		if (kind ~= nil) then
			frame.eDone = 1;
			frame:AddLine("Can disenchant from level "..levelRange.." items.", 0.7,0.7,0.7);
			if (FDP_Config.Verbose) then
				if (kind == "shard") then
					frame:AddLine("Always disenchants from blue or better items, rarely from green items.", 0.7,0.7,0.7);
				elseif (kind == "essence") then
					frame:AddLine("Often disenchants from green weapons, occasionally from other green items.", 0.7,0.7,0.7);
				elseif (kind == "dust") then
					frame:AddLine("Frequently disenchants from any green items.", 0.7,0.7,0.7);
				end
			end
			
			frame:Show();
		end
	end
	
end

function FDP_OnLoad()

	-- Register Slash Commands
	SLASH_FDP1 = "/enchant";
	SLASH_FDP2 = "/disenchant";
	SLASH_FDP3 = "/ench";
	SLASH_FDP4 = "/dis";
	SLASH_FDP5 = "/de";
	SlashCmdList["FDP"] = function(msg)
		FDP_ChatCommandHandler(msg);
	end
	
	FDP_Old_Tooltip_Hook = TT_AddTooltip;
	TT_AddTooltip = FDP_Tooltip_Hook;

	GFWUtils.Print("Fizzwidget Disenchant Predictor initialized!");
end

function FDP_ChatCommandHandler(msg)

	-- Print Help
	if ( msg == "help" ) or ( msg == "" ) then
		GFWUtils.Print("Fizzwidget Disenchant Predictor:");
		GFWUtils.Print("/enchant (or /ench or /disenchant or /dis or /de)");
		GFWUtils.Print("- "..Hilite("help").." - Print this helplist.");
		GFWUtils.Print("- "..Hilite("status").." - Check current settings.");
		GFWUtils.Print("- "..Hilite("tooltip on").." | "..Hilite("off").." - adds a line to tooltips on enchant reagents showing which levels of items they come from.");
		GFWUtils.Print("- "..Hilite("verbose on").." | "..Hilite("off").." - an extra tooltip line showing which types of items disenchant to which reagents.");
		GFWUtils.Print("- "..Hilite("<level number>").." - provide an item's level requirement to see what it can disenchant to.");
		return;
	end
	
	if (msg == "tooltip on") then
		FDP_Config.Tooltip = true;
			GFWUtils.Print("Showing disenchant level range on tooltips for enchanting materials.");
		return;
	end
	if (msg == "tooltip off") then
		FDP_Config.Tooltip = false;
		GFWUtils.Print("Not adding any info to tooltips for enchanting materials.");
		return;
	end
	if (msg == "verbose on") then
		FDP_Config.Verbose = true;
		GFWUtils.Print("Also showing more specific info for certain enchanting reagent tooltips.");
		return;
	end
	if (msg == "verbose off") then
		FDP_Config.Verbose = false;
		GFWUtils.Print("Not showing more specific disenchant info in reagent tooltips.");
		return;
	end
	
	if ( msg == "status" ) then
		if ( FDP_Config.Tooltip ) then
			GFWUtils.Print("Showing disenchant level range on tooltips for enchanting materials.");
			if ( FDP_Config.Verbose ) then
				GFWUtils.Print("Also showing more specific info for certain enchanting reagent tooltips.");
			else
				GFWUtils.Print("Not showing more specific disenchant info.");
			end
		else
			GFWUtils.Print("Not adding any info to tooltips for enchanting materials.");
		end
		return;
	end

	if ( tonumber(msg) ~= nil ) then
		FDP_GreenDisenchantByLevel(tonumber(msg))
		return;
	end
	
	index, _, color, link, name = string.find(msg, "|c(%x+)|Hitem:(%d+:%d+:%d+:%d+)|h%[(.-)%]|h|r");
	if (index) then
		if ( color == FDP_COLOR_EPIC ) then
			return;
		elseif ( color == FDP_COLOR_RARE ) then
			return;
		elseif ( color == FDP_COLOR_UNCOMMON ) then
			return;
		else
			return;
		end
	end
end

function FDP_GreenDisenchantByLevel(level)
	
	dustIndex = math.max(1, math.floor((level - 11) / 10));
	essenceIndex = math.max(1, math.floor((level - 6) / 5));
	shardIndex = math.max(1, math.floor((level - 16) / 5));
	
	GFWUtils.Print("An item of level "..Hilite(level).." can disenchant into:");
	GFWUtils.Print(Hilite(FDP_DUST_LIST[dustIndex]).." (likely for green items)");
	GFWUtils.Print(Hilite(FDP_ESSENCE_LIST[essenceIndex]).." (likely for green weapons, occasionally for other green items)");
	GFWUtils.Print(Hilite(FDP_SHARD_LIST[shardIndex]).." (rarely for green items, always for blue or better items)");
end

function FDP_ItemInfoForLink(itemLink)

end
